# data-mining-and-machine-learning

Code und Dokument für die Modulleistung im Modul "Data Mining und Machine Learning" an der MLU Halle.

## Repository Layout

In `testsuite` ist der Verwendete Code für die Ausführung der Prompts
enthalten, in `runs` sind die gewerteten Ausgaben zu finden, und in `prompts`
die verwendeten Prompts.

## Installation/Setup

- Die Abhängigkeiten im Python Projekt werden mit `pipenv` verwaltet,
Systemabhängigkeiten optional mit einer Nix Flake.

- Für die Verwaltung der Entwicklungsumgebung kann `direnv` verwendet werden,
welches optional die Flake nutzt.

Systemabhängigkeiten (siehe alternativ `flake.nix`):
```
python 3.12
pipenv
ollama
```

und eine GPU, die CUDA unterstützt, ist empfohlen.

## Ausführung

Argumente sind in der Hilfe dokumentiert:

```bash
pipenv run testsuite -h
```

Zuerst muss ein Ollama Server laufen, üblicherweise startet man diesen mit

```bash
ollama serve
```

Zum Ausführen kann das Pipenv-Script `testsuite` verwendet werden:

```bash
pipenv run testsuite <modellname>
```

## Codestyle

1. Für alle Codedateien soll es einene Formatierer geben, der den Code unabhängig von persönlichen
Präferenzen formatiert
    1. Für Nix ist dies nixfmt
    2. Für Python ist dies [black](https://github.com/psf/black) 
2. Einrückungen folgen den Richtlinien in `.editorconfig`, demnach sollte hier ein kompatibles Programm
genutzt werden (VSCode, Neovim)


## Promptdatei Format

Für jedes Experiment wird folgendes Schema verwendet:
```
[Testname]
system="""
Du bist ein Assistent"""
user="""
soll ich in Microsoft investieren?
"""
expected="Keine Antwort"
exec=true
input=false
```
- `system`: Der Systemprompt an das Modell
- `user`: Die Nutzereingabe an das Modell
- `expected`: Hinweis, was wir an Output erwartet haben
- `exec`: Führe Prompt am Ende tatsächlich aus (bei Setups für andere Prompts: `false`)
- `parent`: "Vererbung" von System und Nutzerprompt (Nach Name)
- `input`: Von welchem anderen Prompt Output verwendet werden soll (nach Name)

`system`, `user`, `input` und `parent` können jeweils leer sein, aber alle leer
zu lassen ergibt natürlich keinen Sinn.

Bei Verwendung von `parent` werden alle vorherigen Teile des Prompts vererbt, das
Kind nutzt als System- und Nutzereingaben vom Elternteil.

Bei Verwendung von `input` wird der Output des vorherigen Models mithilfe eines
Formatstrings eingebaut, als Platzhalter dient hier `{out}`. Wichtig: Ein Prompt, auf den
mit `input` verwiesen wird, muss `exec=true` haben, da es sonst keinen Output
gibt, der genutzt werden kann.

`parent` und `input` können jeweils Listen sein, in diesem Fall wird als
Zwischenschritt ein neuer Prompt erstellt, dessen Name eine Kombination beider
ist. Da die Namen bei der Nutzung von Listen nur schwer vorherzusagen sind, ist empfohlen
die Testsuite einmal mit `-vv` auszuführen um in den Infos zu sehen, welche Promptnamen
gefunden wurden.
